2008-04-10 09:41:13 +00:00
// This file is part of Eigen, a lightweight C++ template library
2009-05-22 20:25:33 +02:00
// for linear algebra.
2008-04-10 09:41:13 +00:00
//
2015-07-29 10:22:25 +02:00
// Copyright (C) 2008-2015 Gael Guennebaud <gael.guennebaud@inria.fr>
2008-11-24 13:40:43 +00:00
// Copyright (C) 2006-2008 Benoit Jacob <jacob.benoit.1@gmail.com>
2008-04-10 09:41:13 +00:00
//
2012-07-13 14:42:47 -04:00
// This Source Code Form is subject to the terms of the Mozilla
// Public License v. 2.0. If a copy of the MPL was not distributed
// with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
2008-04-10 09:41:13 +00:00
# ifndef EIGEN_MACROS_H
# define EIGEN_MACROS_H
2018-07-12 16:57:41 +02:00
//------------------------------------------------------------------------------------------
// Eigen version and basic defaults
//------------------------------------------------------------------------------------------
2011-03-18 05:27:58 -04:00
# define EIGEN_WORLD_VERSION 3
2021-02-17 23:23:17 +01:00
# define EIGEN_MAJOR_VERSION 4
# define EIGEN_MINOR_VERSION 99
2009-01-04 16:19:12 +00:00
# define EIGEN_VERSION_AT_LEAST(x,y,z) (EIGEN_WORLD_VERSION>x || (EIGEN_WORLD_VERSION>=x && \
( EIGEN_MAJOR_VERSION > y | | ( EIGEN_MAJOR_VERSION > = y & & \
EIGEN_MINOR_VERSION > = z ) ) ) )
2014-11-04 21:58:52 +01:00
2018-07-12 16:57:41 +02:00
# ifdef EIGEN_DEFAULT_TO_ROW_MAJOR
# define EIGEN_DEFAULT_MATRIX_STORAGE_ORDER_OPTION Eigen::RowMajor
# else
# define EIGEN_DEFAULT_MATRIX_STORAGE_ORDER_OPTION Eigen::ColMajor
# endif
# ifndef EIGEN_DEFAULT_DENSE_INDEX_TYPE
# define EIGEN_DEFAULT_DENSE_INDEX_TYPE std::ptrdiff_t
# endif
// Upperbound on the C++ version to use.
// Expected values are 03, 11, 14, 17, etc.
// By default, let's use an arbitrarily large C++ version.
# ifndef EIGEN_MAX_CPP_VER
# define EIGEN_MAX_CPP_VER 99
# endif
/** Allows to disable some optimizations which might affect the accuracy of the result.
* Such optimization are enabled by default , and set EIGEN_FAST_MATH to 0 to disable them .
* They currently include :
* - single precision ArrayBase : : sin ( ) and ArrayBase : : cos ( ) for SSE and AVX vectorization .
*/
# ifndef EIGEN_FAST_MATH
# define EIGEN_FAST_MATH 1
# endif
# ifndef EIGEN_STACK_ALLOCATION_LIMIT
// 131072 == 128 KB
2021-03-05 14:29:37 -08:00
# define EIGEN_STACK_ALLOCATION_LIMIT 131072
2018-07-12 16:57:41 +02:00
# endif
//------------------------------------------------------------------------------------------
2014-11-04 21:58:52 +01:00
// Compiler identification, EIGEN_COMP_*
2018-07-12 16:57:41 +02:00
//------------------------------------------------------------------------------------------
2014-11-04 21:58:52 +01:00
/// \internal EIGEN_COMP_GNUC set to 1 for all compilers compatible with GCC
2010-03-06 09:05:15 -05:00
# ifdef __GNUC__
2017-02-01 23:36:40 +01:00
# define EIGEN_COMP_GNUC (__GNUC__*10+__GNUC_MINOR__)
2010-03-06 09:05:15 -05:00
# else
2014-11-04 21:58:52 +01:00
# define EIGEN_COMP_GNUC 0
2010-03-06 09:05:15 -05:00
# endif
2014-11-04 21:58:52 +01:00
2016-08-22 15:37:05 +02:00
/// \internal EIGEN_COMP_CLANG set to major+minor version (e.g., 307 for clang 3.7) if the compiler is clang
2014-11-04 21:58:52 +01:00
# if defined(__clang__)
2016-08-22 15:37:05 +02:00
# define EIGEN_COMP_CLANG (__clang_major__*100+__clang_minor__)
2011-02-16 08:50:19 -05:00
# else
2014-11-04 21:58:52 +01:00
# define EIGEN_COMP_CLANG 0
2011-02-16 08:50:19 -05:00
# endif
2020-09-16 13:37:27 -04:00
/// \internal EIGEN_COMP_CASTXML set to 1 if being preprocessed by CastXML
# if defined(__castxml__)
# define EIGEN_COMP_CASTXML 1
# else
# define EIGEN_COMP_CASTXML 0
# endif
2014-11-04 21:58:52 +01:00
/// \internal EIGEN_COMP_LLVM set to 1 if the compiler backend is llvm
# if defined(__llvm__)
# define EIGEN_COMP_LLVM 1
2011-02-16 08:50:19 -05:00
# else
2014-11-04 21:58:52 +01:00
# define EIGEN_COMP_LLVM 0
# endif
/// \internal EIGEN_COMP_ICC set to __INTEL_COMPILER if the compiler is Intel compiler, 0 otherwise
# if defined(__INTEL_COMPILER)
# define EIGEN_COMP_ICC __INTEL_COMPILER
# else
# define EIGEN_COMP_ICC 0
# endif
/// \internal EIGEN_COMP_MINGW set to 1 if the compiler is mingw
# if defined(__MINGW32__)
# define EIGEN_COMP_MINGW 1
# else
# define EIGEN_COMP_MINGW 0
# endif
/// \internal EIGEN_COMP_SUNCC set to 1 if the compiler is Solaris Studio
# if defined(__SUNPRO_CC)
# define EIGEN_COMP_SUNCC 1
# else
# define EIGEN_COMP_SUNCC 0
# endif
/// \internal EIGEN_COMP_MSVC set to _MSC_VER if the compiler is Microsoft Visual C++, 0 otherwise.
# if defined(_MSC_VER)
# define EIGEN_COMP_MSVC _MSC_VER
# else
# define EIGEN_COMP_MSVC 0
# endif
2019-05-31 15:26:06 -07:00
# if defined(__NVCC__)
# if defined(__CUDACC_VER_MAJOR__) && (__CUDACC_VER_MAJOR__ >= 9)
# define EIGEN_COMP_NVCC ((__CUDACC_VER_MAJOR__ * 10000) + (__CUDACC_VER_MINOR__ * 100))
# elif defined(__CUDACC_VER__)
# define EIGEN_COMP_NVCC __CUDACC_VER__
# else
# error "NVCC did not define compiler version."
# endif
# else
# define EIGEN_COMP_NVCC 0
# endif
2016-07-18 15:16:13 +02:00
// For the record, here is a table summarizing the possible values for EIGEN_COMP_MSVC:
2019-05-03 14:14:09 -07:00
// name ver MSC_VER
2018-04-04 15:14:09 +02:00
// 2008 9 1500
// 2010 10 1600
// 2012 11 1700
// 2013 12 1800
// 2015 14 1900
// "15" 15 1900
// 2017-14.1 15.0 1910
// 2017-14.11 15.3 1911
// 2017-14.12 15.5 1912
// 2017-14.13 15.6 1913
// 2017-14.14 15.7 1914
2016-07-18 15:16:13 +02:00
2019-05-03 14:14:09 -07:00
/// \internal EIGEN_COMP_MSVC_LANG set to _MSVC_LANG if the compiler is Microsoft Visual C++, 0 otherwise.
# if defined(_MSVC_LANG)
# define EIGEN_COMP_MSVC_LANG _MSVC_LANG
# else
# define EIGEN_COMP_MSVC_LANG 0
# endif
// For the record, here is a table summarizing the possible values for EIGEN_COMP_MSVC_LANG:
// MSVC option Standard MSVC_LANG
// /std:c++14 (default as of VS 2019) C++14 201402L
// /std:c++17 C++17 201703L
// /std:c++latest >C++17 >201703L
2017-02-10 13:41:52 +01:00
/// \internal EIGEN_COMP_MSVC_STRICT set to 1 if the compiler is really Microsoft Visual C++ and not ,e.g., ICC or clang-cl
# if EIGEN_COMP_MSVC && !(EIGEN_COMP_ICC || EIGEN_COMP_LLVM || EIGEN_COMP_CLANG)
2014-12-18 22:58:15 +01:00
# define EIGEN_COMP_MSVC_STRICT _MSC_VER
2014-11-04 21:58:52 +01:00
# else
# define EIGEN_COMP_MSVC_STRICT 0
# endif
2019-01-11 17:10:54 +01:00
/// \internal EIGEN_COMP_IBM set to xlc version if the compiler is IBM XL C++
// XLC version
// 3.1 0x0301
// 4.5 0x0405
// 5.0 0x0500
// 12.1 0x0C01
# if defined(__IBMCPP__) || defined(__xlc__) || defined(__ibmxl__)
# define EIGEN_COMP_IBM __xlC__
2014-11-04 21:58:52 +01:00
# else
# define EIGEN_COMP_IBM 0
# endif
2019-01-11 17:10:54 +01:00
/// \internal EIGEN_COMP_PGI set to PGI version if the compiler is Portland Group Compiler
2014-11-04 21:58:52 +01:00
# if defined(__PGI)
2019-01-11 17:10:54 +01:00
# define EIGEN_COMP_PGI (__PGIC__*100+__PGIC_MINOR__)
2014-11-04 21:58:52 +01:00
# else
# define EIGEN_COMP_PGI 0
2011-02-16 08:50:19 -05:00
# endif
2010-03-06 09:05:15 -05:00
2014-11-04 21:58:52 +01:00
/// \internal EIGEN_COMP_ARM set to 1 if the compiler is ARM Compiler
# if defined(__CC_ARM) || defined(__ARMCC_VERSION)
# define EIGEN_COMP_ARM 1
# else
# define EIGEN_COMP_ARM 0
# endif
2019-03-21 02:21:04 +01:00
/// \internal EIGEN_COMP_EMSCRIPTEN set to 1 if the compiler is Emscripten Compiler
2016-02-10 12:48:34 -05:00
# if defined(__EMSCRIPTEN__)
2016-02-10 11:11:49 -05:00
# define EIGEN_COMP_EMSCRIPTEN 1
# else
# define EIGEN_COMP_EMSCRIPTEN 0
# endif
2014-11-04 21:58:52 +01:00
/// \internal EIGEN_GNUC_STRICT set to 1 if the compiler is really GCC and not a compatible compiler (e.g., ICC, clang, mingw, etc.)
2016-02-10 11:11:49 -05:00
# if EIGEN_COMP_GNUC && !(EIGEN_COMP_CLANG || EIGEN_COMP_ICC || EIGEN_COMP_MINGW || EIGEN_COMP_PGI || EIGEN_COMP_IBM || EIGEN_COMP_ARM || EIGEN_COMP_EMSCRIPTEN)
2014-11-04 21:58:52 +01:00
# define EIGEN_COMP_GNUC_STRICT 1
# else
# define EIGEN_COMP_GNUC_STRICT 0
# endif
# if EIGEN_COMP_GNUC
# define EIGEN_GNUC_AT_LEAST(x,y) ((__GNUC__==x && __GNUC_MINOR__>=y) || __GNUC__>x)
# define EIGEN_GNUC_AT_MOST(x,y) ((__GNUC__==x && __GNUC_MINOR__<=y) || __GNUC__<x)
# define EIGEN_GNUC_AT(x,y) ( __GNUC__==x && __GNUC_MINOR__==y )
# else
# define EIGEN_GNUC_AT_LEAST(x,y) 0
# define EIGEN_GNUC_AT_MOST(x,y) 0
# define EIGEN_GNUC_AT(x,y) 0
# endif
// FIXME: could probably be removed as we do not support gcc 3.x anymore
# if EIGEN_COMP_GNUC && (__GNUC__ <= 3)
2010-03-06 09:05:15 -05:00
# define EIGEN_GCC3_OR_OLDER 1
# else
# define EIGEN_GCC3_OR_OLDER 0
# endif
2009-01-04 16:19:12 +00:00
2014-11-04 21:58:52 +01:00
2018-07-12 16:57:41 +02:00
//------------------------------------------------------------------------------------------
2014-11-04 21:58:52 +01:00
// Architecture identification, EIGEN_ARCH_*
2018-07-12 16:57:41 +02:00
//------------------------------------------------------------------------------------------
2014-11-04 21:58:52 +01:00
2021-03-08 19:43:38 +00:00
# if defined(__x86_64__) || (defined(_M_X64) && !defined(_M_ARM64EC)) || defined(__amd64)
2014-11-04 21:58:52 +01:00
# define EIGEN_ARCH_x86_64 1
# else
# define EIGEN_ARCH_x86_64 0
# endif
# if defined(__i386__) || defined(_M_IX86) || defined(_X86_) || defined(__i386)
# define EIGEN_ARCH_i386 1
# else
# define EIGEN_ARCH_i386 0
# endif
# if EIGEN_ARCH_x86_64 || EIGEN_ARCH_i386
# define EIGEN_ARCH_i386_OR_x86_64 1
# else
# define EIGEN_ARCH_i386_OR_x86_64 0
# endif
/// \internal EIGEN_ARCH_ARM set to 1 if the architecture is ARM
# if defined(__arm__)
# define EIGEN_ARCH_ARM 1
# else
# define EIGEN_ARCH_ARM 0
# endif
/// \internal EIGEN_ARCH_ARM64 set to 1 if the architecture is ARM64
2021-03-08 19:43:38 +00:00
# if defined(__aarch64__) || defined(_M_ARM64) || defined(_M_ARM64EC)
2014-11-04 21:58:52 +01:00
# define EIGEN_ARCH_ARM64 1
# else
# define EIGEN_ARCH_ARM64 0
# endif
2020-10-28 20:15:09 +00:00
/// \internal EIGEN_ARCH_ARM_OR_ARM64 set to 1 if the architecture is ARM or ARM64
2015-01-30 17:27:56 -05:00
# if EIGEN_ARCH_ARM || EIGEN_ARCH_ARM64
# define EIGEN_ARCH_ARM_OR_ARM64 1
# else
# define EIGEN_ARCH_ARM_OR_ARM64 0
# endif
2021-02-25 14:29:49 -08:00
/// \internal EIGEN_ARCH_ARMV8 set to 1 if the architecture is armv8 or greater.
# if EIGEN_ARCH_ARM_OR_ARM64 && defined(__ARM_ARCH) && __ARM_ARCH >= 8
# define EIGEN_ARCH_ARMV8 1
# else
# define EIGEN_ARCH_ARMV8 0
# endif
2020-10-28 20:15:09 +00:00
/// \internal EIGEN_HAS_ARM64_FP16 set to 1 if the architecture provides an IEEE
/// compliant Arm fp16 type
# if EIGEN_ARCH_ARM64
# ifndef EIGEN_HAS_ARM64_FP16
# if defined(__ARM_FP16_FORMAT_IEEE)
# define EIGEN_HAS_ARM64_FP16 1
# else
# define EIGEN_HAS_ARM64_FP16 0
# endif
# endif
# endif
/// \internal EIGEN_HAS_ARM64_FP16_VECTOR_ARITHMETIC set to 1 if the architecture
/// supports Neon vector intrinsics for fp16.
# if EIGEN_ARCH_ARM64
# ifndef EIGEN_HAS_ARM64_FP16_VECTOR_ARITHMETIC
# if defined(__ARM_FEATURE_FP16_VECTOR_ARITHMETIC)
# define EIGEN_HAS_ARM64_FP16_VECTOR_ARITHMETIC 1
# else
# define EIGEN_HAS_ARM64_FP16_VECTOR_ARITHMETIC 0
# endif
# endif
# endif
/// \internal EIGEN_HAS_ARM64_FP16_SCALAR_ARITHMETIC set to 1 if the architecture
/// supports Neon scalar intrinsics for fp16.
# if EIGEN_ARCH_ARM64
# ifndef EIGEN_HAS_ARM64_FP16_SCALAR_ARITHMETIC
# if defined(__ARM_FEATURE_FP16_SCALAR_ARITHMETIC)
# define EIGEN_HAS_ARM64_FP16_SCALAR_ARITHMETIC 1
# endif
# endif
# endif
2014-11-04 21:58:52 +01:00
/// \internal EIGEN_ARCH_MIPS set to 1 if the architecture is MIPS
# if defined(__mips__) || defined(__mips)
# define EIGEN_ARCH_MIPS 1
# else
# define EIGEN_ARCH_MIPS 0
# endif
/// \internal EIGEN_ARCH_SPARC set to 1 if the architecture is SPARC
# if defined(__sparc__) || defined(__sparc)
# define EIGEN_ARCH_SPARC 1
# else
# define EIGEN_ARCH_SPARC 0
# endif
/// \internal EIGEN_ARCH_IA64 set to 1 if the architecture is Intel Itanium
# if defined(__ia64__)
# define EIGEN_ARCH_IA64 1
# else
# define EIGEN_ARCH_IA64 0
# endif
/// \internal EIGEN_ARCH_PPC set to 1 if the architecture is PowerPC
# if defined(__powerpc__) || defined(__ppc__) || defined(_M_PPC)
# define EIGEN_ARCH_PPC 1
# else
# define EIGEN_ARCH_PPC 0
# endif
2018-07-12 16:57:41 +02:00
//------------------------------------------------------------------------------------------
2014-11-04 21:58:52 +01:00
// Operating system identification, EIGEN_OS_*
2018-07-12 16:57:41 +02:00
//------------------------------------------------------------------------------------------
2014-11-04 21:58:52 +01:00
/// \internal EIGEN_OS_UNIX set to 1 if the OS is a unix variant
# if defined(__unix__) || defined(__unix)
# define EIGEN_OS_UNIX 1
# else
# define EIGEN_OS_UNIX 0
# endif
/// \internal EIGEN_OS_LINUX set to 1 if the OS is based on Linux kernel
# if defined(__linux__)
# define EIGEN_OS_LINUX 1
# else
# define EIGEN_OS_LINUX 0
# endif
/// \internal EIGEN_OS_ANDROID set to 1 if the OS is Android
2015-03-31 11:17:21 -04:00
// note: ANDROID is defined when using ndk_build, __ANDROID__ is defined when using a standalone toolchain.
# if defined(__ANDROID__) || defined(ANDROID)
2014-11-04 21:58:52 +01:00
# define EIGEN_OS_ANDROID 1
# else
# define EIGEN_OS_ANDROID 0
# endif
/// \internal EIGEN_OS_GNULINUX set to 1 if the OS is GNU Linux and not Linux-based OS (e.g., not android)
# if defined(__gnu_linux__) && !(EIGEN_OS_ANDROID)
# define EIGEN_OS_GNULINUX 1
# else
# define EIGEN_OS_GNULINUX 0
# endif
/// \internal EIGEN_OS_BSD set to 1 if the OS is a BSD variant
# if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__) || defined(__bsdi__) || defined(__DragonFly__)
# define EIGEN_OS_BSD 1
# else
# define EIGEN_OS_BSD 0
# endif
/// \internal EIGEN_OS_MAC set to 1 if the OS is MacOS
# if defined(__APPLE__)
# define EIGEN_OS_MAC 1
# else
# define EIGEN_OS_MAC 0
# endif
/// \internal EIGEN_OS_QNX set to 1 if the OS is QNX
# if defined(__QNX__)
# define EIGEN_OS_QNX 1
# else
# define EIGEN_OS_QNX 0
# endif
/// \internal EIGEN_OS_WIN set to 1 if the OS is Windows based
# if defined(_WIN32)
# define EIGEN_OS_WIN 1
# else
# define EIGEN_OS_WIN 0
# endif
/// \internal EIGEN_OS_WIN64 set to 1 if the OS is Windows 64bits
# if defined(_WIN64)
# define EIGEN_OS_WIN64 1
# else
# define EIGEN_OS_WIN64 0
# endif
/// \internal EIGEN_OS_WINCE set to 1 if the OS is Windows CE
# if defined(_WIN32_WCE)
# define EIGEN_OS_WINCE 1
# else
# define EIGEN_OS_WINCE 0
# endif
/// \internal EIGEN_OS_CYGWIN set to 1 if the OS is Windows/Cygwin
# if defined(__CYGWIN__)
# define EIGEN_OS_CYGWIN 1
# else
# define EIGEN_OS_CYGWIN 0
# endif
/// \internal EIGEN_OS_WIN_STRICT set to 1 if the OS is really Windows and not some variants
# if EIGEN_OS_WIN && !( EIGEN_OS_WINCE || EIGEN_OS_CYGWIN )
# define EIGEN_OS_WIN_STRICT 1
# else
# define EIGEN_OS_WIN_STRICT 0
# endif
2019-01-11 17:10:54 +01:00
/// \internal EIGEN_OS_SUN set to __SUNPRO_C if the OS is SUN
// compiler solaris __SUNPRO_C
// version studio
// 5.7 10 0x570
// 5.8 11 0x580
// 5.9 12 0x590
// 5.10 12.1 0x5100
// 5.11 12.2 0x5110
// 5.12 12.3 0x5120
2015-04-24 11:26:51 +02:00
# if (defined(sun) || defined(__sun)) && !(defined(__SVR4) || defined(__svr4__))
2019-01-11 17:10:54 +01:00
# define EIGEN_OS_SUN __SUNPRO_C
2015-04-24 11:26:51 +02:00
# else
# define EIGEN_OS_SUN 0
# endif
/// \internal EIGEN_OS_SOLARIS set to 1 if the OS is Solaris
# if (defined(sun) || defined(__sun)) && (defined(__SVR4) || defined(__svr4__))
# define EIGEN_OS_SOLARIS 1
# else
# define EIGEN_OS_SOLARIS 0
# endif
2014-11-04 21:58:52 +01:00
2018-07-12 16:57:41 +02:00
//------------------------------------------------------------------------------------------
// Detect GPU compilers and architectures
//------------------------------------------------------------------------------------------
2014-11-04 21:58:52 +01:00
2018-07-12 16:57:41 +02:00
// NVCC is not supported as the target platform for HIPCC
// Note that this also makes EIGEN_CUDACC and EIGEN_HIPCC mutually exclusive
# if defined(__NVCC__) && defined(__HIPCC__)
# error "NVCC as the target platform for HIPCC is currently not supported."
2014-11-04 21:58:52 +01:00
# endif
2018-07-12 16:57:41 +02:00
# if defined(__CUDACC__) && !defined(EIGEN_NO_CUDA)
// Means the compiler is either nvcc or clang with CUDA enabled
# define EIGEN_CUDACC __CUDACC__
# endif
2014-10-20 15:55:32 +02:00
2018-07-12 16:57:41 +02:00
# if defined(__CUDA_ARCH__) && !defined(EIGEN_NO_CUDA)
// Means we are generating code for the device
# define EIGEN_CUDA_ARCH __CUDA_ARCH__
# endif
2019-05-31 15:26:06 -07:00
# if defined(EIGEN_CUDACC)
# include <cuda.h>
# define EIGEN_CUDA_SDK_VER (CUDA_VERSION * 10)
2009-01-04 15:26:32 +00:00
# else
2019-05-31 15:26:06 -07:00
# define EIGEN_CUDA_SDK_VER 0
2009-01-04 15:26:32 +00:00
# endif
2018-07-12 16:57:41 +02:00
# if defined(__HIPCC__) && !defined(EIGEN_NO_HIP)
// Means the compiler is HIPCC (analogous to EIGEN_CUDACC, but for HIP)
# define EIGEN_HIPCC __HIPCC__
2018-07-17 14:16:48 -04:00
// We need to include hip_runtime.h here because it pulls in
2018-07-18 16:32:55 -07:00
// ++ hip_common.h which contains the define for __HIP_DEVICE_COMPILE__
2018-07-17 14:16:48 -04:00
// ++ host_defines.h which contains the defines for the __host__ and __device__ macros
# include <hip/hip_runtime.h>
2018-07-12 16:57:41 +02:00
# if defined(__HIP_DEVICE_COMPILE__)
// analogous to EIGEN_CUDA_ARCH, but for HIP
# define EIGEN_HIP_DEVICE_COMPILE __HIP_DEVICE_COMPILE__
# endif
2020-08-19 20:06:39 +00:00
// For HIP (ROCm 3.5 and higher), we need to explicitly set the launch_bounds attribute
// value to 1024. The compiler assigns a default value of 256 when the attribute is not
// specified. This results in failures on the HIP platform, for cases when a GPU kernel
// without an explicit launch_bounds attribute is called with a threads_per_block value
// greater than 256.
//
// This is a regression in functioanlity and is expected to be fixed within the next
// couple of ROCm releases (compiler will go back to using 1024 value as the default)
//
// In the meantime, we will use a "only enabled for HIP" macro to set the launch_bounds
// attribute.
# define EIGEN_HIP_LAUNCH_BOUNDS_1024 __launch_bounds__(1024)
2018-07-12 16:57:41 +02:00
# endif
2020-08-19 20:06:39 +00:00
# if !defined(EIGEN_HIP_LAUNCH_BOUNDS_1024)
# define EIGEN_HIP_LAUNCH_BOUNDS_1024
# endif // !defined(EIGEN_HIP_LAUNCH_BOUNDS_1024)
2018-07-12 16:57:41 +02:00
// Unify CUDA/HIPCC
# if defined(EIGEN_CUDACC) || defined(EIGEN_HIPCC)
//
// If either EIGEN_CUDACC or EIGEN_HIPCC is defined, then define EIGEN_GPUCC
//
# define EIGEN_GPUCC
//
// EIGEN_HIPCC implies the HIP compiler and is used to tweak Eigen code for use in HIP kernels
// EIGEN_CUDACC implies the CUDA compiler and is used to tweak Eigen code for use in CUDA kernels
//
// In most cases the same tweaks are required to the Eigen code to enable in both the HIP and CUDA kernels.
// For those cases, the corresponding code should be guarded with
// #if defined(EIGEN_GPUCC)
// instead of
// #if defined(EIGEN_CUDACC) || defined(EIGEN_HIPCC)
//
// For cases where the tweak is specific to HIP, the code should be guarded with
// #if defined(EIGEN_HIPCC)
//
// For cases where the tweak is specific to CUDA, the code should be guarded with
// #if defined(EIGEN_CUDACC)
//
# endif
# if defined(EIGEN_CUDA_ARCH) || defined(EIGEN_HIP_DEVICE_COMPILE)
//
// If either EIGEN_CUDA_ARCH or EIGEN_HIP_DEVICE_COMPILE is defined, then define EIGEN_GPU_COMPILE_PHASE
//
# define EIGEN_GPU_COMPILE_PHASE
//
// GPU compilers (HIPCC, NVCC) typically do two passes over the source code,
// + one to compile the source for the "host" (ie CPU)
// + another to compile the source for the "device" (ie. GPU)
//
// Code that needs to enabled only during the either the "host" or "device" compilation phase
// needs to be guarded with a macro that indicates the current compilation phase
//
// EIGEN_HIP_DEVICE_COMPILE implies the device compilation phase in HIP
// EIGEN_CUDA_ARCH implies the device compilation phase in CUDA
//
// In most cases, the "host" / "device" specific code is the same for both HIP and CUDA
// For those cases, the code should be guarded with
// #if defined(EIGEN_GPU_COMPILE_PHASE)
// instead of
// #if defined(EIGEN_CUDA_ARCH) || defined(EIGEN_HIP_DEVICE_COMPILE)
//
// For cases where the tweak is specific to HIP, the code should be guarded with
// #if defined(EIGEN_HIP_DEVICE_COMPILE)
//
// For cases where the tweak is specific to CUDA, the code should be guarded with
// #if defined(EIGEN_CUDA_ARCH)
//
# endif
2019-06-27 12:25:09 +01:00
# if defined(EIGEN_USE_SYCL) && defined(__SYCL_DEVICE_ONLY__)
// EIGEN_USE_SYCL is a user-defined macro while __SYCL_DEVICE_ONLY__ is a compiler-defined macro.
// In most cases we want to check if both macros are defined which can be done using the define below.
# define SYCL_DEVICE_ONLY
# endif
2018-07-12 16:57:41 +02:00
//------------------------------------------------------------------------------------------
// Detect Compiler/Architecture/OS specific features
//------------------------------------------------------------------------------------------
# if EIGEN_GNUC_AT_MOST(4,3) && !EIGEN_COMP_CLANG
// see bug 89
# define EIGEN_SAFE_TO_USE_STANDARD_ASSERT_MACRO 0
# else
# define EIGEN_SAFE_TO_USE_STANDARD_ASSERT_MACRO 1
2010-05-30 16:00:58 -04:00
# endif
2014-09-01 10:52:58 +02:00
// Cross compiler wrapper around LLVM's __has_builtin
# ifdef __has_builtin
# define EIGEN_HAS_BUILTIN(x) __has_builtin(x)
# else
# define EIGEN_HAS_BUILTIN(x) 0
# endif
2013-08-02 19:59:43 +02:00
// A Clang feature extension to determine compiler features.
// We use it to determine 'cxx_rvalue_references'
# ifndef __has_feature
# define __has_feature(x) 0
# endif
2017-02-11 10:34:18 +01:00
// Some old compilers do not support template specializations like:
// template<typename T,int N> void foo(const T x[N]);
2018-11-09 10:33:17 +01:00
# if !( EIGEN_COMP_CLANG && ( (EIGEN_COMP_CLANG<309) \
| | ( defined ( __apple_build_version__ ) & & ( __apple_build_version__ < 9000000 ) ) ) \
| | EIGEN_COMP_GNUC_STRICT & & EIGEN_COMP_GNUC < 49 )
2017-02-11 10:34:18 +01:00
# define EIGEN_HAS_STATIC_ARRAY_TEMPLATE 1
2017-02-10 13:08:49 -08:00
# else
2017-02-11 10:34:18 +01:00
# define EIGEN_HAS_STATIC_ARRAY_TEMPLATE 0
2017-02-10 13:08:49 -08:00
# endif
2021-03-01 15:57:22 -08:00
// The macro EIGEN_CPLUSPLUS is a replacement for __cplusplus/_MSVC_LANG that
// works for both platforms, indicating the C++ standard version number.
//
// With MSVC, without defining /Zc:__cplusplus, the __cplusplus macro will
// report 199711L regardless of the language standard specified via /std.
// We need to rely on _MSVC_LANG instead, which is only available after
// VS2015.3.
# if EIGEN_COMP_MSVC_LANG > 0
# define EIGEN_CPLUSPLUS EIGEN_COMP_MSVC_LANG
# elif EIGEN_COMP_MSVC >= 1900
# define EIGEN_CPLUSPLUS 201103L
# elif defined(__cplusplus)
# define EIGEN_CPLUSPLUS __cplusplus
# else
# define EIGEN_CPLUSPLUS 0
# endif
2019-02-19 14:04:35 +01:00
// The macro EIGEN_COMP_CXXVER defines the c++ verson expected by the compiler.
// For instance, if compiling with gcc and -std=c++17, then EIGEN_COMP_CXXVER
// is defined to 17.
2021-03-01 15:57:22 -08:00
# if EIGEN_CPLUSPLUS > 201703L
# define EIGEN_COMP_CXXVER 20
# elif EIGEN_CPLUSPLUS > 201402L
# define EIGEN_COMP_CXXVER 17
# elif EIGEN_CPLUSPLUS > 201103L
# define EIGEN_COMP_CXXVER 14
# elif EIGEN_CPLUSPLUS >= 201103L
# define EIGEN_COMP_CXXVER 11
2019-02-19 14:04:35 +01:00
# else
2021-03-01 15:57:22 -08:00
# define EIGEN_COMP_CXXVER 03
2019-02-19 14:04:35 +01:00
# endif
// The macros EIGEN_HAS_CXX?? defines a rough estimate of available c++ features
// but in practice we should not rely on them but rather on the availabilty of
// individual features as defined later.
// This is why there is no EIGEN_HAS_CXX17.
// FIXME: get rid of EIGEN_HAS_CXX14 and maybe even EIGEN_HAS_CXX11.
# if EIGEN_MAX_CPP_VER>=11 && EIGEN_COMP_CXXVER>=11
2016-10-26 15:53:13 +02:00
# define EIGEN_HAS_CXX11 1
# else
# define EIGEN_HAS_CXX11 0
# endif
2019-02-19 14:04:35 +01:00
# if EIGEN_MAX_CPP_VER>=14 && EIGEN_COMP_CXXVER>=14
2017-01-16 16:13:37 +01:00
# define EIGEN_HAS_CXX14 1
# else
# define EIGEN_HAS_CXX14 0
# endif
2016-10-26 15:53:13 +02:00
2013-08-02 19:59:43 +02:00
// Do we support r-value references?
2016-05-20 14:54:20 +02:00
# ifndef EIGEN_HAS_RVALUE_REFERENCES
2016-05-20 14:44:28 +02:00
# if EIGEN_MAX_CPP_VER>=11 && \
( __has_feature ( cxx_rvalue_references ) | | \
2021-03-01 15:57:22 -08:00
( EIGEN_COMP_CXXVER > = 11 ) | | ( EIGEN_COMP_MSVC > = 1600 ) )
2016-05-20 14:54:20 +02:00
# define EIGEN_HAS_RVALUE_REFERENCES 1
# else
# define EIGEN_HAS_RVALUE_REFERENCES 0
# endif
2013-08-02 19:59:43 +02:00
# endif
2015-12-10 13:52:05 -08:00
// Does the compiler support C99?
2018-07-24 22:56:15 +02:00
// Need to include <cmath> to make sure _GLIBCXX_USE_C99 gets defined
# include <cmath>
2016-05-20 14:58:19 +02:00
# ifndef EIGEN_HAS_C99_MATH
2016-05-20 14:44:28 +02:00
# if EIGEN_MAX_CPP_VER>=11 && \
( ( defined ( __STDC_VERSION__ ) & & ( __STDC_VERSION__ > = 199901 ) ) \
2015-12-10 13:52:05 -08:00
| | ( defined ( __GNUC__ ) & & defined ( _GLIBCXX_USE_C99 ) ) \
2016-11-18 14:18:34 -08:00
| | ( defined ( _LIBCPP_VERSION ) & & ! defined ( _MSC_VER ) ) \
2019-06-27 12:25:09 +01:00
| | ( EIGEN_COMP_MSVC > = 1900 ) | | defined ( SYCL_DEVICE_ONLY ) )
2016-05-20 14:58:19 +02:00
# define EIGEN_HAS_C99_MATH 1
# else
# define EIGEN_HAS_C99_MATH 0
# endif
2015-12-10 13:52:05 -08:00
# endif
2015-02-19 15:18:37 +01:00
// Does the compiler support result_of?
2021-01-07 10:12:25 +00:00
// result_of was deprecated in c++17 and removed in c++ 20
2016-05-20 15:01:27 +02:00
# ifndef EIGEN_HAS_STD_RESULT_OF
2021-01-07 10:12:25 +00:00
# if EIGEN_MAX_CPP_VER >= 11 && \
2021-02-17 11:59:33 -08:00
( defined ( __cplusplus ) & & __cplusplus > = 201103L & & __cplusplus < 201703L )
2015-02-19 15:18:37 +01:00
# define EIGEN_HAS_STD_RESULT_OF 1
2016-05-20 15:01:27 +02:00
# else
# define EIGEN_HAS_STD_RESULT_OF 0
# endif
2021-01-07 12:32:41 +01:00
# endif
2015-02-19 15:18:37 +01:00
2021-02-19 08:52:31 -08:00
// Does the compiler support std::hash?
# ifndef EIGEN_HAS_STD_HASH
// The std::hash struct is defined in C++11 but is not labelled as a __device__
// function and is not constexpr, so cannot be used on device.
2021-02-23 19:52:08 -08:00
# if EIGEN_HAS_CXX11 && !defined(EIGEN_GPU_COMPILE_PHASE)
2021-02-19 08:52:31 -08:00
# define EIGEN_HAS_STD_HASH 1
# else
# define EIGEN_HAS_STD_HASH 0
# endif
# endif // EIGEN_HAS_STD_HASH
2021-02-17 11:59:33 -08:00
# ifndef EIGEN_HAS_STD_INVOKE_RESULT
# if EIGEN_MAX_CPP_VER >= 17 && \
( defined ( __cplusplus ) & & __cplusplus > = 201703L )
# define EIGEN_HAS_STD_INVOKE_RESULT 1
# else
# define EIGEN_HAS_STD_INVOKE_RESULT 0
# endif
# endif
2019-01-11 17:10:54 +01:00
# ifndef EIGEN_HAS_ALIGNAS
# if EIGEN_MAX_CPP_VER>=11 && EIGEN_HAS_CXX11 && \
( __has_feature ( cxx_alignas ) \
| | EIGEN_HAS_CXX14 \
| | ( EIGEN_COMP_MSVC > = 1800 ) \
| | ( EIGEN_GNUC_AT_LEAST ( 4 , 8 ) ) \
| | ( EIGEN_COMP_CLANG > = 305 ) \
| | ( EIGEN_COMP_ICC > = 1500 ) \
| | ( EIGEN_COMP_PGI > = 1500 ) \
2019-01-15 11:04:37 +01:00
| | ( EIGEN_COMP_SUNCC > = 0x5130 ) )
2019-01-11 17:10:54 +01:00
# define EIGEN_HAS_ALIGNAS 1
# else
# define EIGEN_HAS_ALIGNAS 0
# endif
# endif
2018-07-17 00:11:20 +02:00
// Does the compiler support type_traits?
2018-07-23 16:51:57 +02:00
// - full support of type traits was added only to GCC 5.1.0.
// - 20150626 corresponds to the last release of 4.x libstdc++
2017-09-07 14:34:30 +02:00
# ifndef EIGEN_HAS_TYPE_TRAITS
2018-07-18 16:32:55 -07:00
# if EIGEN_MAX_CPP_VER>=11 && (EIGEN_HAS_CXX11 || EIGEN_COMP_MSVC >= 1700) \
2018-07-23 16:51:57 +02:00
& & ( ( ! EIGEN_COMP_GNUC_STRICT ) | | EIGEN_GNUC_AT_LEAST ( 5 , 1 ) ) \
& & ( ( ! defined ( __GLIBCXX__ ) ) | | __GLIBCXX__ > 20150626 )
2017-09-07 14:34:30 +02:00
# define EIGEN_HAS_TYPE_TRAITS 1
# define EIGEN_INCLUDE_TYPE_TRAITS
# else
# define EIGEN_HAS_TYPE_TRAITS 0
# endif
# endif
2014-04-28 10:32:27 -07:00
// Does the compiler support variadic templates?
2016-05-20 15:05:38 +02:00
# ifndef EIGEN_HAS_VARIADIC_TEMPLATES
2021-03-01 15:57:22 -08:00
# if EIGEN_MAX_CPP_VER>=11 && (EIGEN_COMP_CXXVER >= 11) \
2019-05-31 15:26:06 -07:00
& & ( ! defined ( __NVCC__ ) | | ! EIGEN_ARCH_ARM_OR_ARM64 | | ( EIGEN_COMP_NVCC > = 80000 ) )
2016-10-08 22:19:03 +00:00
// ^^ Disable the use of variadic templates when compiling with versions of nvcc older than 8.0 on ARM devices:
2016-05-20 15:05:38 +02:00
// this prevents nvcc from crashing when compiling Eigen on Tegra X1
2014-04-28 10:32:27 -07:00
# define EIGEN_HAS_VARIADIC_TEMPLATES 1
2021-03-01 15:57:22 -08:00
# elif EIGEN_MAX_CPP_VER>=11 && (EIGEN_COMP_CXXVER >= 11) && defined(SYCL_DEVICE_ONLY)
2016-11-25 16:19:07 +00:00
# define EIGEN_HAS_VARIADIC_TEMPLATES 1
2016-05-20 15:05:38 +02:00
# else
# define EIGEN_HAS_VARIADIC_TEMPLATES 0
2014-04-28 10:32:27 -07:00
# endif
2016-02-09 02:34:02 +00:00
# endif
2014-04-28 10:32:27 -07:00
2016-05-20 14:44:28 +02:00
// Does the compiler fully support const expressions? (as in c++14)
2016-05-20 15:10:08 +02:00
# ifndef EIGEN_HAS_CONSTEXPR
2018-07-12 16:57:41 +02:00
# if defined(EIGEN_CUDACC)
// Const expressions are supported provided that c++11 is enabled and we're using either clang or nvcc 7.5 or above
2021-03-01 15:57:22 -08:00
# if EIGEN_MAX_CPP_VER>=14 && (EIGEN_COMP_CXXVER >= 11 && (EIGEN_COMP_CLANG || EIGEN_COMP_NVCC >= 70500))
2018-08-01 12:36:24 -07:00
# define EIGEN_HAS_CONSTEXPR 1
# endif
2021-03-01 15:57:22 -08:00
# elif EIGEN_MAX_CPP_VER>=14 && (__has_feature(cxx_relaxed_constexpr) || (EIGEN_COMP_CXXVER >= 14) || \
( EIGEN_GNUC_AT_LEAST ( 4 , 8 ) & & ( EIGEN_COMP_CXXVER > = 11 ) ) | | \
( EIGEN_COMP_CLANG > = 306 & & ( EIGEN_COMP_CXXVER > = 11 ) ) )
2018-08-01 12:36:24 -07:00
# define EIGEN_HAS_CONSTEXPR 1
2018-07-12 16:57:41 +02:00
# endif
2015-02-06 02:51:59 -08:00
2018-07-12 16:57:41 +02:00
# ifndef EIGEN_HAS_CONSTEXPR
2018-08-01 12:36:24 -07:00
# define EIGEN_HAS_CONSTEXPR 0
2018-07-12 16:57:41 +02:00
# endif
2016-05-20 15:10:08 +02:00
2018-07-12 16:57:41 +02:00
# endif // EIGEN_HAS_CONSTEXPR
2016-05-20 15:10:08 +02:00
2020-09-18 17:38:58 +00:00
# if EIGEN_HAS_CONSTEXPR
# define EIGEN_CONSTEXPR constexpr
# else
# define EIGEN_CONSTEXPR
# endif
2015-06-11 21:45:02 +02:00
// Does the compiler support C++11 math?
2015-06-10 16:26:55 +02:00
// Let's be conservative and enable the default C++11 implementation only if we are sure it exists
2015-06-11 21:45:02 +02:00
# ifndef EIGEN_HAS_CXX11_MATH
2021-03-01 15:57:22 -08:00
# if EIGEN_MAX_CPP_VER>=11 && ((EIGEN_COMP_CXXVER > 11) || (EIGEN_COMP_CXXVER == 11) && (EIGEN_COMP_GNUC_STRICT || EIGEN_COMP_CLANG || EIGEN_COMP_MSVC || EIGEN_COMP_ICC) \
2016-05-20 14:44:28 +02:00
& & ( EIGEN_ARCH_i386_OR_x86_64 ) & & ( EIGEN_OS_GNULINUX | | EIGEN_OS_WIN_STRICT | | EIGEN_OS_MAC ) )
2015-06-11 21:45:02 +02:00
# define EIGEN_HAS_CXX11_MATH 1
# else
# define EIGEN_HAS_CXX11_MATH 0
# endif
2015-04-20 13:14:06 -04:00
# endif
2015-02-06 02:51:59 -08:00
2015-08-14 16:09:48 +02:00
// Does the compiler support proper C++11 containers?
# ifndef EIGEN_HAS_CXX11_CONTAINERS
2016-05-20 14:44:28 +02:00
# if EIGEN_MAX_CPP_VER>=11 && \
2021-03-01 15:57:22 -08:00
( ( EIGEN_COMP_CXXVER > 11 ) \
| | ( ( EIGEN_COMP_CXXVER = = 11 ) & & ( EIGEN_COMP_GNUC_STRICT | | EIGEN_COMP_CLANG | | EIGEN_COMP_MSVC | | EIGEN_COMP_ICC > = 1400 ) ) )
2015-08-14 16:09:48 +02:00
# define EIGEN_HAS_CXX11_CONTAINERS 1
# else
# define EIGEN_HAS_CXX11_CONTAINERS 0
# endif
# endif
2015-12-10 14:21:23 +01:00
// Does the compiler support C++11 noexcept?
# ifndef EIGEN_HAS_CXX11_NOEXCEPT
2016-05-20 14:44:28 +02:00
# if EIGEN_MAX_CPP_VER>=11 && \
2016-06-03 14:28:25 +02:00
( __has_feature ( cxx_noexcept ) \
2021-03-01 15:57:22 -08:00
| | ( EIGEN_COMP_CXXVER > 11 ) \
| | ( ( EIGEN_COMP_CXXVER = = 11 ) & & ( EIGEN_COMP_GNUC_STRICT | | EIGEN_COMP_CLANG | | EIGEN_COMP_MSVC | | EIGEN_COMP_ICC > = 1400 ) ) )
2015-12-10 14:21:23 +01:00
# define EIGEN_HAS_CXX11_NOEXCEPT 1
# else
# define EIGEN_HAS_CXX11_NOEXCEPT 0
# endif
# endif
2018-07-17 00:11:20 +02:00
# ifndef EIGEN_HAS_CXX11_ATOMIC
# if EIGEN_MAX_CPP_VER>=11 && \
( __has_feature ( cxx_atomic ) \
2021-03-01 15:57:22 -08:00
| | ( EIGEN_COMP_CXXVER > 11 ) \
| | ( ( EIGEN_COMP_CXXVER = = 11 ) & & ( EIGEN_COMP_MSVC = = 0 | | EIGEN_COMP_MSVC > = 1700 ) ) )
2018-07-17 00:11:20 +02:00
# define EIGEN_HAS_CXX11_ATOMIC 1
# else
# define EIGEN_HAS_CXX11_ATOMIC 0
# endif
# endif
2018-07-12 16:57:41 +02:00
2018-09-24 18:01:17 +02:00
# ifndef EIGEN_HAS_CXX11_OVERRIDE_FINAL
# if EIGEN_MAX_CPP_VER>=11 && \
2021-03-01 15:57:22 -08:00
( EIGEN_COMP_CXXVER > = 11 | | EIGEN_COMP_MSVC > = 1700 )
2018-09-24 18:01:17 +02:00
# define EIGEN_HAS_CXX11_OVERRIDE_FINAL 1
# else
# define EIGEN_HAS_CXX11_OVERRIDE_FINAL 0
# endif
# endif
2019-02-20 13:52:11 +01:00
// NOTE: the required Apple's clang version is very conservative
// and it could be that XCode 9 works just fine.
// NOTE: the MSVC version is based on https://en.cppreference.com/w/cpp/compiler_support
// and not tested.
# ifndef EIGEN_HAS_CXX17_OVERALIGN
# if EIGEN_MAX_CPP_VER>=17 && EIGEN_COMP_CXXVER>=17 && ( \
( EIGEN_COMP_MSVC > = 1912 ) \
| | ( EIGEN_GNUC_AT_LEAST ( 7 , 0 ) ) \
| | ( ( ! defined ( __apple_build_version__ ) ) & & ( EIGEN_COMP_CLANG > = 500 ) ) \
| | ( ( defined ( __apple_build_version__ ) ) & & ( __apple_build_version__ > = 10000000 ) ) \
)
# define EIGEN_HAS_CXX17_OVERALIGN 1
# else
# define EIGEN_HAS_CXX17_OVERALIGN 0
# endif
# endif
2018-07-12 16:57:41 +02:00
# if defined(EIGEN_CUDACC) && EIGEN_HAS_CONSTEXPR
// While available already with c++11, this is useful mostly starting with c++14 and relaxed constexpr rules
# if defined(__NVCC__)
// nvcc considers constexpr functions as __host__ __device__ with the option --expt-relaxed-constexpr
# ifdef __CUDACC_RELAXED_CONSTEXPR__
# define EIGEN_CONSTEXPR_ARE_DEVICE_FUNC
# endif
2018-10-18 16:55:02 -07:00
# elif defined(__clang__) && defined(__CUDA__) && __has_feature(cxx_relaxed_constexpr)
// clang++ always considers constexpr functions as implicitly __host__ __device__
# define EIGEN_CONSTEXPR_ARE_DEVICE_FUNC
2018-07-12 16:57:41 +02:00
# endif
2009-03-26 12:50:24 +00:00
# endif
2019-11-06 14:24:33 -08:00
// Does the compiler support the __int128 and __uint128_t extensions for 128-bit
// integer arithmetic?
//
// Clang and GCC define __SIZEOF_INT128__ when these extensions are supported,
// but we avoid using them in certain cases:
//
// * Building using Clang for Windows, where the Clang runtime library has
// 128-bit support only on LP64 architectures, but Windows is LLP64.
# ifndef EIGEN_HAS_BUILTIN_INT128
# if defined(__SIZEOF_INT128__) && !(EIGEN_OS_WIN && EIGEN_COMP_CLANG)
# define EIGEN_HAS_BUILTIN_INT128 1
# else
# define EIGEN_HAS_BUILTIN_INT128 0
# endif
# endif
2018-07-12 16:57:41 +02:00
//------------------------------------------------------------------------------------------
// Preprocessor programming helpers
//------------------------------------------------------------------------------------------
// This macro can be used to prevent from macro expansion, e.g.:
// std::max EIGEN_NOT_A_MACRO(a,b)
# define EIGEN_NOT_A_MACRO
2009-08-11 16:12:34 -04:00
# define EIGEN_DEBUG_VAR(x) std::cerr << #x << " = " << x << std::endl;
2011-02-16 08:50:19 -05:00
// concatenate two tokens
# define EIGEN_CAT2(a,b) a ## b
# define EIGEN_CAT(a,b) EIGEN_CAT2(a,b)
2008-04-10 09:41:13 +00:00
2016-06-14 15:06:35 +02:00
# define EIGEN_COMMA ,
2011-02-16 08:50:19 -05:00
// convert a token to a string
# define EIGEN_MAKESTRING2(a) #a
# define EIGEN_MAKESTRING(a) EIGEN_MAKESTRING2(a)
2008-04-10 09:41:13 +00:00
2011-11-06 16:27:41 -05:00
// EIGEN_STRONG_INLINE is a stronger version of the inline, using __forceinline on MSVC,
// but it still doesn't use GCC's always_inline. This is useful in (common) situations where MSVC needs forceinline
// but GCC is still doing fine with just inline.
2017-12-14 14:47:38 +01:00
# ifndef EIGEN_STRONG_INLINE
2021-03-11 11:23:00 -08:00
# if (EIGEN_COMP_MSVC || EIGEN_COMP_ICC) && !defined(EIGEN_GPUCC)
2008-12-19 15:38:39 +00:00
# define EIGEN_STRONG_INLINE __forceinline
2008-04-10 09:41:13 +00:00
# else
2009-01-19 08:21:32 +00:00
# define EIGEN_STRONG_INLINE inline
2008-04-10 09:41:13 +00:00
# endif
2017-12-14 14:47:38 +01:00
# endif
2008-04-10 09:41:13 +00:00
2011-11-06 16:27:41 -05:00
// EIGEN_ALWAYS_INLINE is the stronget, it has the effect of making the function inline and adding every possible
// attribute to maximize inlining. This should only be used when really necessary: in particular,
// it uses __attribute__((always_inline)) on GCC, which most of the time is useless and can severely harm compile times.
// FIXME with the always_inline attribute,
2016-12-20 22:17:01 +01:00
// gcc 3.4.x and 4.1 reports the following compilation error:
2011-11-06 16:27:41 -05:00
// Eval.h:91: sorry, unimplemented: inlining failed in call to 'const Eigen::Eval<Derived> Eigen::MatrixBase<Scalar, Derived>::eval() const'
// : function body not available
2016-12-20 22:17:01 +01:00
// See also bug 1367
2019-06-27 12:25:09 +01:00
# if EIGEN_GNUC_AT_LEAST(4,2) && !defined(SYCL_DEVICE_ONLY)
2011-11-06 16:27:41 -05:00
# define EIGEN_ALWAYS_INLINE __attribute__((always_inline)) inline
# else
# define EIGEN_ALWAYS_INLINE EIGEN_STRONG_INLINE
# endif
2014-11-04 21:58:52 +01:00
# if EIGEN_COMP_GNUC
2008-05-05 17:19:47 +00:00
# define EIGEN_DONT_INLINE __attribute__((noinline))
2014-11-04 21:58:52 +01:00
# elif EIGEN_COMP_MSVC
2008-12-18 13:36:31 +00:00
# define EIGEN_DONT_INLINE __declspec(noinline)
2008-05-05 17:19:47 +00:00
# else
# define EIGEN_DONT_INLINE
# endif
2014-11-04 21:58:52 +01:00
# if EIGEN_COMP_GNUC
2013-01-23 20:51:38 +01:00
# define EIGEN_PERMISSIVE_EXPR __extension__
# else
# define EIGEN_PERMISSIVE_EXPR
# endif
2018-07-12 16:57:41 +02:00
// GPU stuff
// Disable some features when compiling with GPU compilers (NVCC/clang-cuda/SYCL/HIPCC)
2019-06-27 12:25:09 +01:00
# if defined(EIGEN_CUDACC) || defined(SYCL_DEVICE_ONLY) || defined(EIGEN_HIPCC)
2018-07-12 16:57:41 +02:00
// Do not try asserts on device code
# ifndef EIGEN_NO_DEBUG
# define EIGEN_NO_DEBUG
# endif
# ifdef EIGEN_INTERNAL_DEBUGGING
# undef EIGEN_INTERNAL_DEBUGGING
# endif
# ifdef EIGEN_EXCEPTIONS
# undef EIGEN_EXCEPTIONS
# endif
# endif
2019-06-27 12:25:09 +01:00
# if defined(SYCL_DEVICE_ONLY)
# ifndef EIGEN_DONT_VECTORIZE
# define EIGEN_DONT_VECTORIZE
# endif
2019-11-28 10:08:54 +00:00
# define EIGEN_DEVICE_FUNC __attribute__((flatten)) __attribute__((always_inline))
2018-07-12 16:57:41 +02:00
// All functions callable from CUDA/HIP code must be qualified with __device__
2019-06-27 12:25:09 +01:00
# elif defined(EIGEN_GPUCC)
# define EIGEN_DEVICE_FUNC __host__ __device__
2018-07-12 16:57:41 +02:00
# else
# define EIGEN_DEVICE_FUNC
# endif
2010-06-18 09:01:34 -04:00
// this macro allows to get rid of linking errors about multiply defined functions.
// - static is not very good because it prevents definitions from different object files to be merged.
// So static causes the resulting linked executable to be bloated with multiple copies of the same function.
// - inline is not perfect either as it unwantedly hints the compiler toward inlining the function.
2017-02-28 17:05:28 -08:00
# define EIGEN_DECLARE_FUNCTION_ALLOWING_MULTIPLE_DEFINITIONS EIGEN_DEVICE_FUNC
# define EIGEN_DEFINE_FUNCTION_ALLOWING_MULTIPLE_DEFINITIONS EIGEN_DEVICE_FUNC inline
2010-06-18 09:01:34 -04:00
2011-02-16 08:50:19 -05:00
# ifdef NDEBUG
# ifndef EIGEN_NO_DEBUG
# define EIGEN_NO_DEBUG
# endif
# endif
// eigen_plain_assert is where we implement the workaround for the assert() bug in GCC <= 4.3, see bug 89
# ifdef EIGEN_NO_DEBUG
2019-06-27 12:25:09 +01:00
# ifdef SYCL_DEVICE_ONLY // used to silence the warning on SYCL device
# define eigen_plain_assert(x) EIGEN_UNUSED_VARIABLE(x)
# else
# define eigen_plain_assert(x)
# endif
# else
2011-02-16 08:50:19 -05:00
# if EIGEN_SAFE_TO_USE_STANDARD_ASSERT_MACRO
namespace Eigen {
namespace internal {
inline bool copy_bool ( bool b ) { return b ; }
}
}
# define eigen_plain_assert(x) assert(x)
# else
// work around bug 89
2011-02-17 11:51:48 +00:00
# include <cstdlib> // for abort
# include <iostream> // for std::cerr
2011-02-16 08:50:19 -05:00
namespace Eigen {
namespace internal {
// trivial function copying a bool. Must be EIGEN_DONT_INLINE, so we implement it after including Eigen headers.
// see bug 89.
namespace {
EIGEN_DONT_INLINE bool copy_bool ( bool b ) { return b ; }
}
inline void assert_fail ( const char * condition , const char * function , const char * file , int line )
{
std : : cerr < < " assertion failed: " < < condition < < " in function " < < function < < " at " < < file < < " : " < < line < < std : : endl ;
abort ( ) ;
}
}
}
# define eigen_plain_assert(x) \
do { \
if ( ! Eigen : : internal : : copy_bool ( x ) ) \
Eigen : : internal : : assert_fail ( EIGEN_MAKESTRING ( x ) , __PRETTY_FUNCTION__ , __FILE__ , __LINE__ ) ; \
} while ( false )
# endif
# endif
// eigen_assert can be overridden
# ifndef eigen_assert
# define eigen_assert(x) eigen_plain_assert(x)
# endif
# ifdef EIGEN_INTERNAL_DEBUGGING
# define eigen_internal_assert(x) eigen_assert(x)
# else
# define eigen_internal_assert(x)
# endif
# ifdef EIGEN_NO_DEBUG
2014-10-30 19:59:09 +01:00
# define EIGEN_ONLY_USED_FOR_DEBUG(x) EIGEN_UNUSED_VARIABLE(x)
2011-02-16 08:50:19 -05:00
# else
# define EIGEN_ONLY_USED_FOR_DEBUG(x)
# endif
2011-10-24 09:40:37 +02:00
# ifndef EIGEN_NO_DEPRECATED_WARNING
2014-11-04 21:58:52 +01:00
# if EIGEN_COMP_GNUC
2011-10-24 09:40:37 +02:00
# define EIGEN_DEPRECATED __attribute__((deprecated))
2014-11-04 21:58:52 +01:00
# elif EIGEN_COMP_MSVC
2011-10-24 09:40:37 +02:00
# define EIGEN_DEPRECATED __declspec(deprecated)
# else
# define EIGEN_DEPRECATED
# endif
2008-10-13 15:53:27 +00:00
# else
2011-10-24 09:40:37 +02:00
# define EIGEN_DEPRECATED
2008-10-13 15:53:27 +00:00
# endif
2014-11-04 21:58:52 +01:00
# if EIGEN_COMP_GNUC
2009-05-04 12:49:56 +00:00
# define EIGEN_UNUSED __attribute__((unused))
# else
# define EIGEN_UNUSED
# endif
2010-06-08 15:52:00 +02:00
// Suppresses 'unused variable' warnings.
2014-02-24 18:13:49 +01:00
namespace Eigen {
namespace internal {
2019-02-15 16:35:35 +01:00
template < typename T > EIGEN_DEVICE_FUNC EIGEN_STRONG_INLINE void ignore_unused_variable ( const T & ) { }
2014-02-24 18:13:49 +01:00
}
}
# define EIGEN_UNUSED_VARIABLE(var) Eigen::internal::ignore_unused_variable(var);
2010-06-08 15:52:00 +02:00
2013-06-21 17:49:36 +02:00
# if !defined(EIGEN_ASM_COMMENT)
2015-01-30 17:27:56 -05:00
# if EIGEN_COMP_GNUC && (EIGEN_ARCH_i386_OR_x86_64 || EIGEN_ARCH_ARM_OR_ARM64)
2014-09-06 11:47:24 +01:00
# define EIGEN_ASM_COMMENT(X) __asm__("#" X)
2013-06-21 17:49:36 +02:00
# else
# define EIGEN_ASM_COMMENT(X)
# endif
2009-03-17 13:25:26 +00:00
# endif
2015-07-29 10:22:25 +02:00
2021-03-03 19:22:15 -08:00
// Acts as a barrier preventing operations involving `X` from crossing. This
// occurs, for example, in the fast rounding trick where a magic constant is
// added then subtracted, which is otherwise compiled away with -ffast-math.
//
// See bug 1674
# if !defined(EIGEN_OPTIMIZATION_BARRIER)
# if EIGEN_COMP_GNUC
// According to https://gcc.gnu.org/onlinedocs/gcc/Constraints.html:
// X: Any operand whatsoever.
// r: A register operand is allowed provided that it is in a general
// register.
// g: Any register, memory or immediate integer operand is allowed, except
// for registers that are not general registers.
// w: (AArch32/AArch64) Floating point register, Advanced SIMD vector
// register or SVE vector register.
// x: (SSE) Any SSE register.
// (AArch64) Like w, but restricted to registers 0 to 15 inclusive.
// v: (PowerPC) An Altivec vector register.
// wa:(PowerPC) A VSX register.
//
// "X" (uppercase) should work for all cases, though this seems to fail for
// some versions of GCC for arm/aarch64 with
// "error: inconsistent operand constraints in an 'asm'"
// Clang x86_64/arm/aarch64 seems to require "g" to support both scalars and
// vectors, otherwise
// "error: non-trivial scalar-to-vector conversion, possible invalid
// constraint for vector type"
//
// GCC for ppc64le generates an internal compiler error with x/X/g.
// GCC for AVX generates an internal compiler error with X.
//
// Tested on icc/gcc/clang for sse, avx, avx2, avx512dq
// gcc for arm, aarch64,
// gcc for ppc64le,
// both vectors and scalars.
//
// Note that this is restricted to plain types - this will not work
// directly for std::complex<T>, Eigen::half, Eigen::bfloat16. For these,
// you will need to apply to the underlying POD type.
2021-03-10 16:04:02 -08:00
# if EIGEN_ARCH_PPC && EIGEN_COMP_GNUC_STRICT
// This seems to be broken on clang. Packet4f is loaded into a single
// register rather than a vector, zeroing out some entries. Integer
// types also generate a compile error.
2021-03-03 19:22:15 -08:00
// General, Altivec, VSX.
# define EIGEN_OPTIMIZATION_BARRIER(X) __asm__ ("" : "+r,v,wa" (X));
# elif EIGEN_ARCH_ARM_OR_ARM64
// General, NEON.
# define EIGEN_OPTIMIZATION_BARRIER(X) __asm__ ("" : "+g,w" (X));
# elif EIGEN_ARCH_i386_OR_x86_64
// General, SSE.
# define EIGEN_OPTIMIZATION_BARRIER(X) __asm__ ("" : "+g,x" (X));
# else
// Not implemented for other architectures.
# define EIGEN_OPTIMIZATION_BARRIER(X)
# endif
# else
// Not implemented for other compilers.
# define EIGEN_OPTIMIZATION_BARRIER(X)
# endif
# endif
2016-12-20 22:52:27 +01:00
# if EIGEN_COMP_MSVC
// NOTE MSVC often gives C4127 warnings with compiletime if statements. See bug 1362.
// This workaround is ugly, but it does the job.
# define EIGEN_CONST_CONDITIONAL(cond) (void)0, cond
# else
# define EIGEN_CONST_CONDITIONAL(cond) cond
# endif
2009-09-19 19:45:58 -04:00
# ifdef EIGEN_DONT_USE_RESTRICT_KEYWORD
# define EIGEN_RESTRICT
# endif
# ifndef EIGEN_RESTRICT
# define EIGEN_RESTRICT __restrict
# endif
2008-08-19 11:06:40 +00:00
2008-12-16 15:44:48 +00:00
2008-11-17 12:45:57 +00:00
# ifndef EIGEN_DEFAULT_IO_FORMAT
2011-02-06 12:51:42 -05:00
# ifdef EIGEN_MAKING_DOCS
// format used in Eigen's documentation
// needed to define it here as escaping characters in CMake add_definition's argument seems very problematic.
# define EIGEN_DEFAULT_IO_FORMAT Eigen::IOFormat(3, 0, " ", "\n", "", "")
# else
2008-11-17 12:45:57 +00:00
# define EIGEN_DEFAULT_IO_FORMAT Eigen::IOFormat()
# endif
2011-02-06 12:51:42 -05:00
# endif
2008-11-17 12:45:57 +00:00
2009-01-27 17:40:14 +00:00
// just an empty macro !
# define EIGEN_EMPTY
2018-07-12 16:57:41 +02:00
// When compiling CUDA/HIP device code with NVCC or HIPCC
// pull in math functions from the global namespace.
// In host mode, and when device code is compiled with clang,
// use the std versions.
# if (defined(EIGEN_CUDA_ARCH) && defined(__NVCC__)) || defined(EIGEN_HIP_DEVICE_COMPILE)
2018-11-19 18:13:59 +00:00
# define EIGEN_USING_STD(FUNC) using ::FUNC;
# else
# define EIGEN_USING_STD(FUNC) using std::FUNC;
# endif
2019-05-31 15:26:06 -07:00
# if EIGEN_COMP_MSVC_STRICT && (EIGEN_COMP_MSVC < 1900 || EIGEN_COMP_NVCC)
2017-08-24 11:06:47 +02:00
// for older MSVC versions, as well as 1900 && CUDA 8, using the base operator is sufficient (cf Bugs 1000, 1324)
2014-11-04 21:58:52 +01:00
# define EIGEN_INHERIT_ASSIGNMENT_EQUAL_OPERATOR(Derived) \
using Base : : operator = ;
# elif EIGEN_COMP_CLANG // workaround clang bug (see http://forum.kde.org/viewtopic.php?f=74&t=102653)
# define EIGEN_INHERIT_ASSIGNMENT_EQUAL_OPERATOR(Derived) \
using Base : : operator = ; \
EIGEN_DEVICE_FUNC EIGEN_STRONG_INLINE Derived & operator = ( const Derived & other ) { Base : : operator = ( other ) ; return * this ; } \
template < typename OtherDerived > \
EIGEN_DEVICE_FUNC EIGEN_STRONG_INLINE Derived & operator = ( const DenseBase < OtherDerived > & other ) { Base : : operator = ( other . derived ( ) ) ; return * this ; }
# else
# define EIGEN_INHERIT_ASSIGNMENT_EQUAL_OPERATOR(Derived) \
using Base : : operator = ; \
EIGEN_DEVICE_FUNC EIGEN_STRONG_INLINE Derived & operator = ( const Derived & other ) \
{ \
Base : : operator = ( other ) ; \
return * this ; \
}
2009-09-03 02:50:42 -04:00
# endif
2008-04-10 09:41:13 +00:00
2015-04-23 13:39:03 +02:00
2019-12-19 17:30:11 +01:00
/**
* \ internal
* \ brief Macro to explicitly define the default copy constructor .
* This is necessary , because the implicit definition is deprecated if the copy - assignment is overridden .
*/
# if EIGEN_HAS_CXX11
2020-10-12 21:52:00 -04:00
# define EIGEN_DEFAULT_COPY_CONSTRUCTOR(CLASS) CLASS(const CLASS&) = default;
2019-12-19 17:30:11 +01:00
# else
# define EIGEN_DEFAULT_COPY_CONSTRUCTOR(CLASS)
# endif
2015-04-23 13:39:03 +02:00
/** \internal
* \ brief Macro to manually inherit assignment operators .
* This is necessary , because the implicitly defined assignment operator gets deleted when a custom operator = is defined .
2019-12-19 17:30:11 +01:00
* With C + + 11 or later this also default - implements the copy - constructor
*/
# define EIGEN_INHERIT_ASSIGNMENT_OPERATORS(Derived) \
EIGEN_INHERIT_ASSIGNMENT_EQUAL_OPERATOR ( Derived ) \
EIGEN_DEFAULT_COPY_CONSTRUCTOR ( Derived )
/** \internal
* \ brief Macro to manually define default constructors and destructors .
* This is necessary when the copy constructor is re - defined .
* For empty helper classes this should usually be protected , to avoid accidentally creating empty objects .
*
* Hiding the default destructor lead to problems in C + + 03 mode together with boost : : multiprecision
2015-04-23 13:39:03 +02:00
*/
2019-12-19 17:30:11 +01:00
# if EIGEN_HAS_CXX11
# define EIGEN_DEFAULT_EMPTY_CONSTRUCTOR_AND_DESTRUCTOR(Derived) \
2020-10-12 21:52:00 -04:00
Derived ( ) = default ; \
~ Derived ( ) = default ;
2019-12-19 17:30:11 +01:00
# else
# define EIGEN_DEFAULT_EMPTY_CONSTRUCTOR_AND_DESTRUCTOR(Derived) \
2020-10-12 21:52:00 -04:00
Derived ( ) { } ; \
/* ~Derived() {}; */
2019-12-19 17:30:11 +01:00
# endif
2009-11-13 16:41:51 +01:00
2010-01-15 15:45:07 +01:00
/**
* Just a side note . Commenting within defines works only by documenting
2010-01-16 11:50:09 -05:00
* behind the object ( via ' ! < ' ) . Comments cannot be multi - line and thus
2010-01-15 15:45:07 +01:00
* we have these extra long lines . What is confusing doxygen over here is
2010-01-22 10:15:41 +01:00
* that we use ' \ ' and basically have a bunch of typedefs with their
2010-01-15 15:45:07 +01:00
* documentation in a single line .
* */
2014-03-10 23:24:40 +01:00
# define EIGEN_GENERIC_PUBLIC_INTERFACE(Derived) \
typedef typename Eigen : : internal : : traits < Derived > : : Scalar Scalar ; /*!< \brief Numeric type, e.g. float, double, int or std::complex<float>. */ \
typedef typename Eigen : : NumTraits < Scalar > : : Real RealScalar ; /*!< \brief The underlying numeric type for composed scalar types. \details In cases where Scalar is e.g. std::complex<T>, T were corresponding to RealScalar. */ \
typedef typename Base : : CoeffReturnType CoeffReturnType ; /*!< \brief The return type for coefficient access. \details Depending on whether the object allows direct coefficient access (e.g. for a MatrixXd), this type is either 'const Scalar&' or simply 'Scalar' for objects that do not allow direct coefficient access. */ \
2015-06-19 17:56:39 +02:00
typedef typename Eigen : : internal : : ref_selector < Derived > : : type Nested ; \
2014-03-10 23:24:40 +01:00
typedef typename Eigen : : internal : : traits < Derived > : : StorageKind StorageKind ; \
2014-12-04 22:48:53 +01:00
typedef typename Eigen : : internal : : traits < Derived > : : StorageIndex StorageIndex ; \
2017-01-25 22:54:51 +01:00
enum CompileTimeTraits \
{ RowsAtCompileTime = Eigen : : internal : : traits < Derived > : : RowsAtCompileTime , \
2014-03-10 23:24:40 +01:00
ColsAtCompileTime = Eigen : : internal : : traits < Derived > : : ColsAtCompileTime , \
Flags = Eigen : : internal : : traits < Derived > : : Flags , \
SizeAtCompileTime = Base : : SizeAtCompileTime , \
MaxSizeAtCompileTime = Base : : MaxSizeAtCompileTime , \
2015-10-08 22:06:49 +02:00
IsVectorAtCompileTime = Base : : IsVectorAtCompileTime } ; \
using Base : : derived ; \
using Base : : const_cast_derived ;
2014-03-10 23:24:40 +01:00
2015-10-09 10:21:54 +02:00
// FIXME Maybe the EIGEN_DENSE_PUBLIC_INTERFACE could be removed as importing PacketScalar is rarely needed
2014-03-10 23:24:40 +01:00
# define EIGEN_DENSE_PUBLIC_INTERFACE(Derived) \
2014-12-04 22:48:53 +01:00
EIGEN_GENERIC_PUBLIC_INTERFACE ( Derived ) \
2015-10-09 10:21:54 +02:00
typedef typename Base : : PacketScalar PacketScalar ;
2015-10-08 22:06:49 +02:00
2008-04-10 09:41:13 +00:00
2010-06-11 07:56:50 -04:00
# define EIGEN_PLAIN_ENUM_MIN(a,b) (((int)a <= (int)b) ? (int)a : (int)b)
# define EIGEN_PLAIN_ENUM_MAX(a,b) (((int)a >= (int)b) ? (int)a : (int)b)
2010-06-14 09:05:08 -04:00
// EIGEN_SIZE_MIN_PREFER_DYNAMIC gives the min between compile-time sizes. 0 has absolute priority, followed by 1,
2010-06-11 07:56:50 -04:00
// followed by Dynamic, followed by other finite values. The reason for giving Dynamic the priority over
// finite values is that min(3, Dynamic) should be Dynamic, since that could be anything between 0 and 3.
2010-06-14 09:05:08 -04:00
# define EIGEN_SIZE_MIN_PREFER_DYNAMIC(a,b) (((int)a == 0 || (int)b == 0) ? 0 \
2010-06-11 07:56:50 -04:00
: ( ( int ) a = = 1 | | ( int ) b = = 1 ) ? 1 \
2009-09-03 02:50:42 -04:00
: ( ( int ) a = = Dynamic | | ( int ) b = = Dynamic ) ? Dynamic \
: ( ( int ) a < = ( int ) b ) ? ( int ) a : ( int ) b )
2010-06-11 07:56:50 -04:00
2010-06-14 09:05:08 -04:00
// EIGEN_SIZE_MIN_PREFER_FIXED is a variant of EIGEN_SIZE_MIN_PREFER_DYNAMIC comparing MaxSizes. The difference is that finite values
2010-06-11 07:56:50 -04:00
// now have priority over Dynamic, so that min(3, Dynamic) gives 3. Indeed, whatever the actual value is
// (between 0 and 3), it is not more than 3.
2010-06-14 09:05:08 -04:00
# define EIGEN_SIZE_MIN_PREFER_FIXED(a,b) (((int)a == 0 || (int)b == 0) ? 0 \
2010-06-11 07:56:50 -04:00
: ( ( int ) a = = 1 | | ( int ) b = = 1 ) ? 1 \
: ( ( int ) a = = Dynamic & & ( int ) b = = Dynamic ) ? Dynamic \
: ( ( int ) a = = Dynamic ) ? ( int ) b \
: ( ( int ) b = = Dynamic ) ? ( int ) a \
: ( ( int ) a < = ( int ) b ) ? ( int ) a : ( int ) b )
2010-06-14 09:05:08 -04:00
// see EIGEN_SIZE_MIN_PREFER_DYNAMIC. No need for a separate variant for MaxSizes here.
2010-06-11 07:56:50 -04:00
# define EIGEN_SIZE_MAX(a,b) (((int)a == Dynamic || (int)b == Dynamic) ? Dynamic \
: ( ( int ) a > = ( int ) b ) ? ( int ) a : ( int ) b )
2009-07-22 23:12:22 +02:00
# define EIGEN_LOGICAL_XOR(a,b) (((a) || (b)) && !((a) && (b)))
2008-04-10 09:41:13 +00:00
2010-03-09 00:16:07 -05:00
# define EIGEN_IMPLIES(a,b) (!(a) || (b))
2009-12-04 23:17:14 +01:00
2019-12-16 21:33:42 +00:00
# if EIGEN_HAS_BUILTIN(__builtin_expect) || EIGEN_COMP_GNUC
# define EIGEN_PREDICT_FALSE(x) (__builtin_expect(x, false))
# define EIGEN_PREDICT_TRUE(x) (__builtin_expect(false || (x), true))
2019-12-16 22:31:59 +00:00
# else
# define EIGEN_PREDICT_FALSE(x) (x)
# define EIGEN_PREDICT_TRUE(x) (x)
2019-12-16 21:33:42 +00:00
# endif
2016-06-10 15:05:43 +02:00
// the expression type of a standard coefficient wise binary operation
Relax mixing-type constraints for binary coefficient-wise operators:
- Replace internal::scalar_product_traits<A,B> by Eigen::ScalarBinaryOpTraits<A,B,OP>
- Remove the "functor_is_product_like" helper (was pretty ugly)
- Currently, OP is not used, but it is available to the user for fine grained tuning
- Currently, only the following operators have been generalized: *,/,+,-,=,*=,/=,+=,-=
- TODO: generalize all other binray operators (comparisons,pow,etc.)
- TODO: handle "scalar op array" operators (currently only * is handled)
- TODO: move the handling of the "void" scalar type to ScalarBinaryOpTraits
2016-06-06 15:11:41 +02:00
# define EIGEN_CWISE_BINARY_RETURN_TYPE(LHS,RHS,OPNAME) \
2009-12-17 13:37:00 +01:00
CwiseBinaryOp < \
Relax mixing-type constraints for binary coefficient-wise operators:
- Replace internal::scalar_product_traits<A,B> by Eigen::ScalarBinaryOpTraits<A,B,OP>
- Remove the "functor_is_product_like" helper (was pretty ugly)
- Currently, OP is not used, but it is available to the user for fine grained tuning
- Currently, only the following operators have been generalized: *,/,+,-,=,*=,/=,+=,-=
- TODO: generalize all other binray operators (comparisons,pow,etc.)
- TODO: handle "scalar op array" operators (currently only * is handled)
- TODO: move the handling of the "void" scalar type to ScalarBinaryOpTraits
2016-06-06 15:11:41 +02:00
EIGEN_CAT ( EIGEN_CAT ( internal : : scalar_ , OPNAME ) , _op ) < \
2010-10-25 10:15:22 -04:00
typename internal : : traits < LHS > : : Scalar , \
typename internal : : traits < RHS > : : Scalar \
2009-12-17 13:37:00 +01:00
> , \
2011-01-24 08:27:06 -05:00
const LHS , \
const RHS \
2009-12-17 13:37:00 +01:00
>
2016-06-10 15:05:43 +02:00
# define EIGEN_MAKE_CWISE_BINARY_OP(METHOD,OPNAME) \
template < typename OtherDerived > \
EIGEN_DEVICE_FUNC EIGEN_STRONG_INLINE const EIGEN_CWISE_BINARY_RETURN_TYPE ( Derived , OtherDerived , OPNAME ) \
( METHOD ) ( const EIGEN_CURRENT_STORAGE_BASE_CLASS < OtherDerived > & other ) const \
{ \
return EIGEN_CWISE_BINARY_RETURN_TYPE ( Derived , OtherDerived , OPNAME ) ( derived ( ) , other . derived ( ) ) ; \
}
2016-07-20 09:56:39 +02:00
# define EIGEN_SCALAR_BINARY_SUPPORTED(OPNAME,TYPEA,TYPEB) \
( Eigen : : internal : : has_ReturnType < Eigen : : ScalarBinaryOpTraits < TYPEA , TYPEB , EIGEN_CAT ( EIGEN_CAT ( Eigen : : internal : : scalar_ , OPNAME ) , _op ) < TYPEA , TYPEB > > > : : value )
2016-06-14 11:26:57 +02:00
# define EIGEN_EXPR_BINARYOP_SCALAR_RETURN_TYPE(EXPR,SCALAR,OPNAME) \
CwiseBinaryOp < EIGEN_CAT ( EIGEN_CAT ( internal : : scalar_ , OPNAME ) , _op ) < typename internal : : traits < EXPR > : : Scalar , SCALAR > , const EXPR , \
const typename internal : : plain_constant_type < EXPR , SCALAR > : : type >
# define EIGEN_SCALAR_BINARYOP_EXPR_RETURN_TYPE(SCALAR,EXPR,OPNAME) \
CwiseBinaryOp < EIGEN_CAT ( EIGEN_CAT ( internal : : scalar_ , OPNAME ) , _op ) < SCALAR , typename internal : : traits < EXPR > : : Scalar > , \
const typename internal : : plain_constant_type < EXPR , SCALAR > : : type , const EXPR >
2016-07-18 15:17:25 +02:00
// Workaround for MSVC 2010 (see ML thread "patch with compile for for MSVC 2010")
2017-06-09 16:25:18 +02:00
# if EIGEN_COMP_MSVC_STRICT && (EIGEN_COMP_MSVC_STRICT<=1600)
2016-07-18 15:17:25 +02:00
# define EIGEN_MSVC10_WORKAROUND_BINARYOP_RETURN_TYPE(X) typename internal::enable_if<true,X>::type
# else
# define EIGEN_MSVC10_WORKAROUND_BINARYOP_RETURN_TYPE(X) X
# endif
2016-06-14 11:26:57 +02:00
# define EIGEN_MAKE_SCALAR_BINARY_OP_ONTHERIGHT(METHOD,OPNAME) \
2019-02-15 16:35:35 +01:00
template < typename T > EIGEN_DEVICE_FUNC EIGEN_STRONG_INLINE \
2016-07-20 09:56:39 +02:00
EIGEN_MSVC10_WORKAROUND_BINARYOP_RETURN_TYPE ( const EIGEN_EXPR_BINARYOP_SCALAR_RETURN_TYPE ( Derived , typename internal : : promote_scalar_arg < Scalar EIGEN_COMMA T EIGEN_COMMA EIGEN_SCALAR_BINARY_SUPPORTED ( OPNAME , Scalar , T ) > : : type , OPNAME ) ) \
2016-06-14 11:26:57 +02:00
( METHOD ) ( const T & scalar ) const { \
2016-07-20 09:56:39 +02:00
typedef typename internal : : promote_scalar_arg < Scalar , T , EIGEN_SCALAR_BINARY_SUPPORTED ( OPNAME , Scalar , T ) > : : type PromotedT ; \
2016-06-23 14:27:20 +02:00
return EIGEN_EXPR_BINARYOP_SCALAR_RETURN_TYPE ( Derived , PromotedT , OPNAME ) ( derived ( ) , \
typename internal : : plain_constant_type < Derived , PromotedT > : : type ( derived ( ) . rows ( ) , derived ( ) . cols ( ) , internal : : scalar_constant_op < PromotedT > ( scalar ) ) ) ; \
2016-06-14 11:26:57 +02:00
}
# define EIGEN_MAKE_SCALAR_BINARY_OP_ONTHELEFT(METHOD,OPNAME) \
2019-02-15 16:35:35 +01:00
template < typename T > EIGEN_DEVICE_FUNC EIGEN_STRONG_INLINE friend \
2016-07-20 09:56:39 +02:00
EIGEN_MSVC10_WORKAROUND_BINARYOP_RETURN_TYPE ( const EIGEN_SCALAR_BINARYOP_EXPR_RETURN_TYPE ( typename internal : : promote_scalar_arg < Scalar EIGEN_COMMA T EIGEN_COMMA EIGEN_SCALAR_BINARY_SUPPORTED ( OPNAME , T , Scalar ) > : : type , Derived , OPNAME ) ) \
2016-06-14 11:26:57 +02:00
( METHOD ) ( const T & scalar , const StorageBaseType & matrix ) { \
2016-07-20 09:56:39 +02:00
typedef typename internal : : promote_scalar_arg < Scalar , T , EIGEN_SCALAR_BINARY_SUPPORTED ( OPNAME , T , Scalar ) > : : type PromotedT ; \
2016-06-23 14:27:20 +02:00
return EIGEN_SCALAR_BINARYOP_EXPR_RETURN_TYPE ( PromotedT , Derived , OPNAME ) ( \
typename internal : : plain_constant_type < Derived , PromotedT > : : type ( matrix . derived ( ) . rows ( ) , matrix . derived ( ) . cols ( ) , internal : : scalar_constant_op < PromotedT > ( scalar ) ) , matrix . derived ( ) ) ; \
2016-06-14 11:26:57 +02:00
}
# define EIGEN_MAKE_SCALAR_BINARY_OP(METHOD,OPNAME) \
EIGEN_MAKE_SCALAR_BINARY_OP_ONTHELEFT ( METHOD , OPNAME ) \
EIGEN_MAKE_SCALAR_BINARY_OP_ONTHERIGHT ( METHOD , OPNAME )
2018-07-12 16:57:41 +02:00
# if (defined(_CPPUNWIND) || defined(__EXCEPTIONS)) && !defined(EIGEN_CUDA_ARCH) && !defined(EIGEN_EXCEPTIONS) && !defined(EIGEN_USE_SYCL) && !defined(EIGEN_HIP_DEVICE_COMPILE)
# define EIGEN_EXCEPTIONS
# endif
2014-07-22 13:16:44 +02:00
# ifdef EIGEN_EXCEPTIONS
# define EIGEN_THROW_X(X) throw X
# define EIGEN_THROW throw
# define EIGEN_TRY try
# define EIGEN_CATCH(X) catch (X)
# else
2018-06-06 10:12:58 -04:00
# if defined(EIGEN_CUDA_ARCH)
2016-09-14 09:56:11 -07:00
# define EIGEN_THROW_X(X) asm("trap;")
# define EIGEN_THROW asm("trap;")
2018-06-06 10:12:58 -04:00
# elif defined(EIGEN_HIP_DEVICE_COMPILE)
# define EIGEN_THROW_X(X) asm("s_trap 0")
# define EIGEN_THROW asm("s_trap 0")
2015-07-22 12:29:18 +02:00
# else
# define EIGEN_THROW_X(X) std::abort()
# define EIGEN_THROW std::abort()
# endif
2014-07-22 13:16:44 +02:00
# define EIGEN_TRY if (true)
# define EIGEN_CATCH(X) else
# endif
2016-06-03 14:28:25 +02:00
2015-12-10 14:21:23 +01:00
# if EIGEN_HAS_CXX11_NOEXCEPT
2016-06-03 14:28:25 +02:00
# define EIGEN_INCLUDE_TYPE_TRAITS
# define EIGEN_NOEXCEPT noexcept
# define EIGEN_NOEXCEPT_IF(x) noexcept(x)
2015-12-10 14:21:23 +01:00
# define EIGEN_NO_THROW noexcept(true)
# define EIGEN_EXCEPTION_SPEC(X) noexcept(false)
# else
2016-06-03 14:28:25 +02:00
# define EIGEN_NOEXCEPT
# define EIGEN_NOEXCEPT_IF(x)
2015-12-10 14:21:23 +01:00
# define EIGEN_NO_THROW throw()
2019-02-19 14:04:35 +01:00
# if EIGEN_COMP_MSVC || EIGEN_COMP_CXXVER>=17
2018-06-07 15:36:20 +02:00
// MSVC does not support exception specifications (warning C4290),
2019-02-19 14:04:35 +01:00
// and they are deprecated in c++11 anyway. This is even an error in c++17.
2018-06-07 15:36:20 +02:00
# define EIGEN_EXCEPTION_SPEC(X) throw()
# else
# define EIGEN_EXCEPTION_SPEC(X) throw(X)
# endif
2015-12-10 14:21:23 +01:00
# endif
2018-08-01 23:35:34 +02:00
# if EIGEN_HAS_VARIADIC_TEMPLATES
2018-08-01 16:04:44 +01:00
// The all function is used to enable a variadic version of eigen_assert which can take a parameter pack as its input.
2018-08-01 12:19:14 +01:00
namespace Eigen {
namespace internal {
2018-08-01 23:35:34 +02:00
inline bool all ( ) { return true ; }
2018-08-01 16:04:44 +01:00
template < typename T , typename . . . Ts >
bool all ( T t , Ts . . . ts ) { return t & & all ( ts . . . ) ; }
2018-08-01 12:19:14 +01:00
}
}
# endif
2018-09-24 18:01:17 +02:00
# if EIGEN_HAS_CXX11_OVERRIDE_FINAL
// provide override and final specifiers if they are available:
# define EIGEN_OVERRIDE override
# define EIGEN_FINAL final
# else
# define EIGEN_OVERRIDE
# define EIGEN_FINAL
# endif
2018-08-08 11:07:27 +01:00
// Wrapping #pragma unroll in a macro since it is required for SYCL
2019-06-27 12:25:09 +01:00
# if defined(SYCL_DEVICE_ONLY)
2018-08-08 11:07:27 +01:00
# if defined(_MSC_VER)
2018-08-08 11:38:10 +01:00
# define EIGEN_UNROLL_LOOP __pragma(unroll)
2018-08-08 11:07:27 +01:00
# else
# define EIGEN_UNROLL_LOOP _Pragma("unroll")
# endif
# else
# define EIGEN_UNROLL_LOOP
# endif
2018-08-01 12:19:14 +01:00
2008-04-10 09:41:13 +00:00
# endif // EIGEN_MACROS_H